/***************************************************************
 * Name:      L4DDSSMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Hermann Hhne (hoehermann@gmx.de)
 * Created:   2009-01-29
 * Copyright: Hermann Hhne (www.hehoe.de)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#ifdef __BORLANDC__
#pragma hdrstop
#endif //__BORLANDC__

#include "L4DDSSMain.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}


L4DDSSFrame::L4DDSSFrame(wxFrame *frame)
    : GUIFrame(frame)
{
    m_sock = new wxSocketClient();
    rcon = new wxRConSocketClient();
    usercon = false;
    configfile = false;
    textfile = new wxTextFile(wxStandardPaths::Get().GetDataDir()+wxT("/config.txt"));
    if (textfile->Exists() && textfile->Open()) {
        addr.Hostname(textfile->GetLine(0));
        addr.Service(textfile->GetLine(1));
        usercon = (textfile->GetLine(2)==wxT("rcon"));
        if (usercon) SetTitle(wxT("L4D Dedicated Server NetControl (rcon Mode)"));
        configfile = true;
        }
    textfile->Close();
    textfile = new wxTextFile(wxStandardPaths::Get().GetDataDir()+wxT("/commands.txt"));
    if (textfile->Exists() && textfile->Open())
    for (wxString command = textfile->GetFirstLine(); !textfile->Eof(); command = textfile->GetNextLine()) {
        serverCommandSelect->Append(command+wxT(" "));
        }
}

L4DDSSFrame::~L4DDSSFrame()
{
    answershavefocus = false;
    gettingstatus = false;
}

void L4DDSSFrame::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void L4DDSSFrame::OnQuit(wxCommandEvent &event)
{
    Destroy();
}

void L4DDSSFrame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxT("rcon and telnet console replacement\nfor Left 4 Dead servers\ncreated by Mc$core\nuses code by Freman\nfurther information available at\nhttp://www.hehoe.de/mcs\n\n")+wxbuildinfo(long_f);
    wxMessageBox(msg, _("L4D - D.S.N.C."));
}

void L4DDSSFrame::OnConnect(wxCommandEvent &event)
{
    if (!configfile) {
        usercon = (wxID_YES == (new wxMessageDialog(this, wxT("Use rcon?\nYES: Use rcon.\nNO: Use netport"), wxT("No config file"), wxYES_NO))->ShowModal());
        addr.Hostname(wxGetTextFromUser(wxT("Enter hostname or IP:"), wxT("No config file"), wxT("")));
        addr.Service(wxGetNumberFromUser(wxT(""),wxT("Enter port:"), wxT("No config File"),1000,0,65526));
        }
    if (usercon) {
        if (!rcon->IsConnected()) {
            SetTitle(wxT("L4D Dedicated Server NetControl (rcon Mode)"));
            wxString passwd = wxGetPasswordFromUser(wxT("Please enter RCON password for ")+addr.IPAddress(), wxT(""), wxT(""));
            switch (rcon->Connect(addr,passwd)) {
                case wxRCONSOCKET_OK:
                    statusBar->SetStatusText(wxT("RCON connected."));
                    EnableControls(true);
                    break;
                case wxRCONSOCKET_AUTHFAIL:
                    statusBar->SetStatusText(wxT("Authentication failed."));
                    break;
                case wxRCONSOCKET_BANNED:
                    statusBar->SetStatusText(wxT("Your IP is banned."));
                    break;
                default:
                    statusBar->SetStatusText(wxT("Unable to connect."));
                };
                return;
            }
        else  {
            statusBar->SetStatusText(_("Can't connect RCON: Already connected."));
            return;
            }
        }
    else {
        if (m_sock->IsConnected()) {
            statusBar->SetStatusText(_("Can't connect: Already connected."));
            return;
            }
        if (m_sock->Connect(addr)) {
            statusBar->SetStatusText(_("Connected."));
            wxString passwd = wxT("PASS ")+wxGetPasswordFromUser(wxT("Please enter NetCon password for ")+addr.IPAddress(), wxT(""), wxT(""));
            const wxChar *buffer = passwd+wxT("\n");
            unsigned char len = (unsigned char)(wxStrlen(buffer) + 1);
            m_sock->Write(buffer, len);
            EnableControls(true);
            this->Connect(CLIENT_ID,wxEVT_SOCKET,wxSocketEventHandler( L4DDSSFrame::OnSocketInput ));
            m_sock->SetEventHandler(*this,CLIENT_ID);
            m_sock->SetNotify(wxSOCKET_INPUT_FLAG);
            m_sock->Notify(true);
        }
        else statusBar->SetStatusText(_("Unable to connect."));
    }
}

void L4DDSSFrame::OnDisconnect(wxCommandEvent &event)
{
    EnableControls(false);
    statusBar->SetStatusText(_("Disconnecting..."));
    if (usercon) {
        rcon->Close();
        if (!rcon->IsConnected()) {
            statusBar->SetStatusText(_("RCON disconnected."));
            return;
            }
        }
    else {
        m_sock->Close();
        if (!m_sock->IsConnected()) {
            statusBar->SetStatusText(_("Disconnected."));
            return;
            }
    }
}

void L4DDSSFrame::OnSocketInput(wxSocketEvent &event)
{
    wxChar *cbuffer = new wxChar[BUFSIZE];
    wxString sbuffer = wxT("");

    switch (event.GetSocketEvent()) {
    case wxSOCKET_INPUT:
        m_sock->Read(cbuffer, BUFSIZE);
        if (m_sock->LastCount()<BUFSIZE) cbuffer[m_sock->LastCount()] = 0;
        sbuffer = wxString(cbuffer);
        sbuffer.Replace(wxString::FromAscii(""),wxT(""),true);
        if (!answershavefocus) serverAnswers->AppendText(sbuffer);
        //else serverAnswers->SetValue(serverAnswers->GetValue()+sbuffer);
        if (gettingstatus) {
            statusinfo += sbuffer;
            if (statusinfo.Contains(wxT("#end"))) {
                parseStatusInfo(statusinfo);
                }
        }
        break;
    case wxSOCKET_LOST:
        EnableControls(false);
        statusBar->SetStatusText(wxT("Connection lost."));
        break;
    default :
        statusBar->SetStatusText(wxT("WARNING: Unhandled socket event!"));
    }
}

void L4DDSSFrame::parseStatusInfo(wxString statusinfo)
{
    gettingstatus = false;

    int start, end;
    wxString steamid,playername,line;

    start = statusinfo.Find(wxT("hostname: "));
    line = statusinfo.Mid(start);
    line = line.BeforeFirst('\n');
    labelHostname->SetLabel(line);
    statusinfo = statusinfo.Right(statusinfo.Length()-line.Length()-start);

    start = statusinfo.Find(wxT("map     : "));
    line = statusinfo.Mid(start);
    line = line.BeforeFirst('\n');
    labelMap->SetLabel(line);
    statusinfo = statusinfo.Right(statusinfo.Length()-line.Length()-start);

    start = statusinfo.Find(wxT("players : "));
    line = statusinfo.Mid(start);
    line = line.BeforeFirst('\n');
    labelPlayers->SetLabel(line);
    statusinfo = statusinfo.Right(statusinfo.Length()-line.Length()-start);

    start = statusinfo.Find(wxT("# userid name uniqueid connected ping loss state rate adr"));
    line = statusinfo.Mid(start);
    line = line.BeforeFirst('\n');
    statusinfo = statusinfo.Right(statusinfo.Length()-line.Length()-start);

    playerlist->Clear();
    while ((statusinfo.Left(4)!=wxT("#end"))&&(statusinfo.Contains(wxT("#end")))) {
        line = statusinfo.BeforeFirst('\n');
        start = line.Find(wxT("\""));
        end = line.Mid(start+1).Find(wxT("\""));
        playername = line.Mid(start,end+2);
        start = line.Find(wxT("STEAM_"));
        end = line.Mid(start).Find(wxT(" "));
        steamid = line.Mid(start,end);
        if (steamid!=wxT("")) {
            playerlist->Append(steamid+_(" ")+playername);
            EnableControls(true);
            }
        statusinfo = statusinfo.AfterFirst('\n');
        }
}

void L4DDSSFrame::OnGetStatus(wxCommandEvent &event)
{
    if (!m_sock->IsConnected() && !rcon->IsConnected()) {
        statusBar->SetStatusText(_("Can't get status: Not connected."));
        return;
        }
    statusinfo = wxT("");
    serverCommandSelect->SetValue(wxT("status"));
    serverCommandArgument->SetValue(wxT(""));
    gettingstatus = true;
    OnButtonSendClick(event);

}

void L4DDSSFrame::OnButtonSendClick(wxCommandEvent &event)
{
    if (serverCommandSend->IsEnabled()) {
        if (usercon) {
            if (!rcon->IsConnected()) {
                statusBar->SetStatusText(wxT("Can't execute RCON command: Not connected."));
                return;
                }
            statusBar->SetStatusText(wxT("Waiting for answer..."));
            EnableControls(false);
            wxString answer = rcon->RConCommand(serverCommandSelect->GetValue()+serverCommandArgument->GetValue());
            statusBar->SetStatusText(wxT(""));
            EnableControls(true);
            if (gettingstatus) parseStatusInfo(answer);
            if (answer.length() > 32000) serverAnswers->SetValue(serverAnswers->GetValue()+answer);
            else serverAnswers->AppendText(answer);
            }
        else {
            if (!m_sock->IsConnected()) {
                statusBar->SetStatusText(_("Can't execute command: Not connected."));
                return;
                }
            const wxChar *buffer = wxT("\n")+serverCommandSelect->GetValue()+serverCommandArgument->GetValue()+wxT("\n");
            unsigned char len = (unsigned char)((wxStrlen(buffer) + 1) * sizeof(wxChar));
            m_sock->Write(buffer, len);
        }
    }
}

void L4DDSSFrame::EnableControls(boolean b)
{
    serverCommandSend->Enable(b);
}

void L4DDSSFrame::OnServerCommandArgumentKeyUp(wxKeyEvent &event)
{
    wxCommandEvent fakeevent = wxCommandEvent();
    if (event.GetKeyCode()==13) OnButtonSendClick(fakeevent);
}

void L4DDSSFrame::OnServerCommandSelectCombobox( wxCommandEvent& event )
{
    serverCommandArgument->SetValue(wxT(""));
}

void L4DDSSFrame::OnButtonKickClick(wxCommandEvent &event)
{
    wxString steamid = playerlist->GetStringSelection();
    steamid = steamid.BeforeFirst(' ');
    if (!steamid.Contains(wxT("STEAM_"))) statusBar->SetStatusText(wxT("Can't kick: No steamid selected."));
    //else if (wxID_YES == (new wxMessageDialog(this, wxT("Really kick ")+playerlist->GetStringSelection()+wxT("?"), wxT("Kick..."), wxYES_NO))->ShowModal()) {
        serverCommandSelect->SetValue(wxT("kickid "));
        serverCommandArgument->SetValue(steamid+wxT(" kicked by remote console"));
    //    OnButtonSendClick(event);
    //    }
}

void L4DDSSFrame::OnButtonBanClick(wxCommandEvent &event)
{
    wxString steamid = playerlist->GetStringSelection();
    steamid = steamid.BeforeFirst(' ');
    if (!steamid.Contains(wxT("STEAM_"))) statusBar->SetStatusText(wxT("Can't ban: No steamid selected."));
    else
    {
        serverCommandSelect->SetValue(wxT("banid "));
        serverCommandArgument->SetValue(wxT("60 ")+steamid);
    }
}

void L4DDSSFrame::OnServerAnswersKillFocus( wxFocusEvent& event )
{
    answershavefocus = false;
}
void L4DDSSFrame::OnServerAnswersSetFocus( wxFocusEvent& event )
{
    answershavefocus = true;
}

void L4DDSSFrame::OnPlayerlistChoice( wxCommandEvent& event )
{
    buttonBan->Enable(playerlist->GetSelection() != wxNOT_FOUND);
    buttonKick->Enable(playerlist->GetSelection() != wxNOT_FOUND);
}
