#ifndef RCON_H_INCLUDED
#define RCON_H_INCLUDED

#include <wx/wx.h>
#include <wx/socket.h>
#include <wx/dynarray.h>


// further information about the rcon protocol at
// http://developer.valvesoftware.com/wiki/Source_RCON_Protocol

const int32_t SERVERDATA_EXECCOMMAND = 2;
const int32_t SERVERDATA_AUTH = 3;

const int PACKETREADTIMEOUT = 1;
const int GENERALTIMEOUT = 1;

enum { wxRCONSOCKET_OK = wxSOCKET_MEMERR+1,
    wxRCONSOCKET_AUTHFAIL,
    wxRCONSOCKET_NONRCON,
    wxRCONSOCKET_BANNED
    };

// a rcon packet like struct
typedef struct {
    int32_t length;
    int32_t id;
    int32_t cmd;
    wxString string1;
    wxString string2;
    } RConPacket;


WX_DECLARE_OBJARRAY(RConPacket, ArrayOfRConPackets);

// see the docs for all wxSocketClient methods
// http://docs.wxwidgets.org/2.8/wx_wxsocketbase.html
class wxRConSocketClient : protected wxSocketClient {
    private:
    int32_t Id;
    protected:
    ArrayOfRConPackets PacketRead ();
    int32_t Write(int32_t cmd , wxString s1, wxString s2);
    void SendCommand(wxString command);
    int Auth (wxString password);
    public:
    wxRConSocketClient(wxSocketFlags flags = wxSOCKET_NONE);
    int Connect(wxSockAddress& address, wxString password);
    // make mandatory methods visible
    bool IsConnected() { return this->wxSocketClient::IsConnected(); }
    bool Close() { return this->wxSocketClient::Close(); }
    wxString RConCommand(wxString command);
    void SetFlags(wxSocketFlags flags);
    ~wxRConSocketClient();
};

#endif // RCON_H_INCLUDED
